package com.fjhx.xmhjc.service.column.impl;

import cn.hutool.core.util.StrUtil;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuSubSelectDto;
import com.fjhx.xmhjc.entity.column.po.ColumnMenu;
import com.fjhx.xmhjc.entity.column.po.ColumnMenuSub;
import com.fjhx.xmhjc.entity.column.vo.ColumnMenuSubVo;
import com.fjhx.xmhjc.mapper.column.ColumnMenuMapper;
import com.fjhx.xmhjc.service.column.ColumnMenuService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.entity.column.vo.ColumnMenuVo;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;


/**
 * <p>
 * 栏目菜单表 服务实现类
 * </p>
 *
 * @author lqh
 * @since 2023-11-15
 */
@Service
public class ColumnMenuServiceImpl extends ServiceImpl<ColumnMenuMapper, ColumnMenu> implements ColumnMenuService {

    @Resource
    private ColumnMenuSubServiceImpl columnMenuSubService;


    @Override
    public List<ColumnMenuVo> getList(ColumnMenuSelectDto dto) {
        IWrapper<ColumnMenu> wrapper = getWrapper();
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("cm", ColumnMenu::getStatus, dto.getStatus());
        }
        if(StrUtil.isNotBlank(dto.getType())){
            wrapper.eq("cm", ColumnMenu::getType, dto.getType());
        }
        if(StrUtil.isNotBlank(dto.getNeType())){
            wrapper.ne("cm", ColumnMenu::getType, dto.getNeType());
        }
        wrapper.orderByAsc("cm", ColumnMenu::getSort);
        List<ColumnMenuVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<ColumnMenuVo> getPage(ColumnMenuSelectDto dto) {
        IWrapper<ColumnMenu> wrapper = getWrapper();
        wrapper.keyword(dto,new SqlField("cm", ColumnMenu::getName));
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("cm", ColumnMenu::getStatus, dto.getStatus());
        }
        wrapper.orderByAsc("cm", ColumnMenu::getSort);
        Page<ColumnMenuVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public ColumnMenuVo detail(Long id) {
        ColumnMenu ColumnMenu = this.getById(id);
        ColumnMenuVo result = BeanUtil.toBean(ColumnMenu, ColumnMenuVo.class);

        ColumnMenuSubSelectDto columnMenuSubSelectDto = new ColumnMenuSubSelectDto();
        columnMenuSubSelectDto.setColumnId(id);
        List<ColumnMenuSubVo> list = columnMenuSubService.getList(columnMenuSubSelectDto);
        result.setColumnMenuSubList(list);

        return result;
    }

    @Override
    @Transactional
    public void add(ColumnMenuDto columnMenuDto) {
        this.save(columnMenuDto);
        //处理子表
        columnMenuSubService.handleColumnMenuSub(columnMenuDto.getId(),columnMenuDto.getColumnMenuSubList());
    }

    @Override
    public void edit(ColumnMenuDto columnMenuDto) {
        ColumnMenu menu = getById(columnMenuDto.getId());
        //栏目类型1=列表；2=图文；3=外链
        //如果  栏目类型 从 列表=》图文/外链  需要判断所有栏目是否有文章
        if ("1".equals(menu.getType()) && !menu.getType().equals(columnMenuDto.getType())){
            //验证是否能删除
            if (columnMenuSubService.checkDeleteByMenu(menu.getId())){
                //删除子栏目
                columnMenuSubService.lambdaUpdate().eq(ColumnMenuSub::getColumnId,menu.getId()).remove();
                //修改
                this.updateById(columnMenuDto);
            }else {
                throw new RuntimeException("该菜单子栏目存在文章，无法更改栏目类型!如不想显示请禁用");
            }

        }else if ("1".equals(menu.getType()) && menu.getType().equals(columnMenuDto.getType())){
            //如果  栏目类型 从 列表=》列表  需要判断是否有文章
            this.updateById(columnMenuDto);
            //处理子表
            columnMenuSubService.handleColumnMenuSub(columnMenuDto.getId(),columnMenuDto.getColumnMenuSubList());
        }else {
            //如果  栏目类型 从 图文/外链=》图文/外链
            this.updateById(columnMenuDto);

        }





    }

    @Override
    public void delete(Long id) {

        ColumnMenu menu = getById(id);
        //栏目类型1=列表；2=图文；3=外链
        if ("1".equals(menu.getType())){
            if (columnMenuSubService.checkDeleteByMenu(id)){
                this.removeById(id);
                columnMenuSubService.lambdaUpdate().eq(ColumnMenuSub::getColumnId,id).remove();
            }else {
                throw new RuntimeException("该菜单子栏目存在文章，删除失败!如不想显示请禁用");
            }
        }else {
            this.removeById(id);
        }
    }

    @Override
    public List<ColumnMenuVo> getMenuListByOpen(ColumnMenuSelectDto dto) {

        List<ColumnMenuVo> list = getList(dto);

        list.forEach(x->{
            //栏目类型1=列表；2=图文；3=外链
            if ("1".equals(x.getType())){
                x.setUrl(null);
                x.setContent(null);
            }else if ("2".equals(x.getType())){
                x.setUrl(null);
                x.setContent(null);
            }else if ("3".equals(x.getType())){
                x.setContent(null);
            }
        });
        return list;
    }


    /**
     * 用于栏目类型-图文 获取图文内容
     * @param id
     * @return
     */
    @Override
    public ColumnMenuVo getMenuDetailByOpen(Long id) {
        ColumnMenu ColumnMenu = this.getById(id);
        ColumnMenuVo result = BeanUtil.toBean(ColumnMenu, ColumnMenuVo.class);
        return result;
    }

}
