package com.fjhx.xmhjc.service.topic.impl;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.fjhx.xmhjc.entity.topic.po.TopicReplies;
import com.fjhx.xmhjc.entity.topic.vo.TopicContentVo;
import com.fjhx.xmhjc.mapper.topic.TopicRepliesMapper;
import com.fjhx.xmhjc.service.topic.TopicRepliesService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.xmhjc.service.website.WebsiteUsersService;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.entity.topic.vo.TopicRepliesVo;
import com.fjhx.xmhjc.entity.topic.dto.TopicRepliesSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.xmhjc.entity.topic.dto.TopicRepliesDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 主题回复 服务实现类
 * </p>
 *
 * @author hj
 * @since 2023-11-20
 */
@Service
public class TopicRepliesServiceImpl extends ServiceImpl<TopicRepliesMapper, TopicReplies> implements TopicRepliesService {

    @Resource
    private WebsiteUsersService websiteUsersService;

    @Override
    public List<TopicRepliesVo> getList(TopicRepliesSelectDto dto) {
        IWrapper<TopicReplies> wrapper = getWrapper();
        wrapper.orderByDesc("tr", TopicReplies::getId);
        List<TopicRepliesVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<TopicRepliesVo> getPage(TopicRepliesSelectDto dto) {
        IWrapper<TopicReplies> wrapper = getWrapper();
        wrapper.orderByDesc("tr", TopicReplies::getId);
        wrapper.eq(NumberUtil.isValidNumber(dto.getTopicId()), "topic_id", dto.getTopicId());
        Page<TopicRepliesVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<TopicRepliesVo> repliesVos = page.getRecords();
        for (TopicRepliesVo repliesVo : repliesVos) {
            repliesVo.setAuthorName(websiteUsersService.getAuthorName(repliesVo.getAuthor()));
        }
        return page;
    }

    @Override
    public TopicRepliesVo detail(Long id) {
        TopicReplies TopicReplies = this.getById(id);
        TopicRepliesVo result = BeanUtil.toBean(TopicReplies, TopicRepliesVo.class);
        return result;
    }

    @Override
    public void add(TopicRepliesDto topicRepliesDto) {
        this.save(topicRepliesDto);
    }

    @Override
    public void edit(TopicRepliesDto topicRepliesDto) {
        this.updateById(topicRepliesDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public Page<TopicRepliesVo> getPageByOpen(TopicRepliesSelectDto dto) {
        IWrapper<TopicReplies> wrapper = getWrapper();
        wrapper.orderByAsc("tr", TopicReplies::getId);
        wrapper.eq("tr", TopicReplies::getTopicId, dto.getTopicId());
        wrapper.eq(ObjectUtil.isNotNull(dto.getFloorId()), "floor_id", dto.getFloorId());
        Page<TopicRepliesVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public Page<TopicRepliesVo> replyPage(TopicRepliesSelectDto dto) {
        IWrapper<TopicReplies> wrapper = getWrapper();
        wrapper.orderByAsc("tr", TopicReplies::getId);
        wrapper.eq("tr", TopicReplies::getTopicId, dto.getTopicId());
        wrapper.eq(ObjectUtil.isNotNull(dto.getFloorId()), "floor_id", dto.getFloorId());
        Page<TopicRepliesVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public Boolean hasReply(Long topicId) {
        Boolean hasReply = false;
        Long count = lambdaQuery().eq(TopicReplies::getTopicId, topicId).count();
        if (count > 0) {
            hasReply = true;
        }
        return hasReply;
    }

    @Override
    public Boolean hasFloorReply(Long floorId) {
        Boolean hasReply = false;
        Long count = lambdaQuery().eq(TopicReplies::getFloorId, floorId).count();
        if (count > 0) {
            hasReply = true;
        }
        return hasReply;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteAndChild(Long id) {
        removeById(id);
        lambdaUpdate().eq(TopicReplies::getFloorId, id).remove();
    }

    @Override
    public Page<TopicRepliesVo> getFloorPageByOpen(TopicRepliesSelectDto dto) {
        Page<TopicRepliesVo> page = this.baseMapper.getFloorPageByOpen(dto.getPage(), dto);
        return page;
    }

    @Override
    public TopicReplies getTopReplies(Long topicId) {
        TopicReplies topReplies = lambdaQuery().eq(TopicReplies::getTopicId, topicId).eq(TopicReplies::getToping, "1").one();
        return topReplies;
    }

    @Override
    public TopicReplies getNewReplies(Long topicId) {
        TopicReplies topReplies = lambdaQuery().eq(TopicReplies::getTopicId, topicId).orderByDesc(TopicReplies::getId).last("limit 1").one();
        return topReplies;
    }

    @Override
    @Transactional
    public void topingReplys(TopicRepliesDto topicRepliesDto) {
        if (StrUtil.equals(topicRepliesDto.getToping(), "1")) {
            //置顶前先把其他的置顶取消
            lambdaUpdate().eq(TopicReplies::getTopicId, topicRepliesDto.getTopicId()).set(TopicReplies::getToping, "0").update();
        }
        updateById(topicRepliesDto);
    }
}
